@@ -4,7 +4,7 @@ RDF::Trine::Store::Hexastore - RDF store implemented with the hexastore index
 
 =head1 VERSION
 
-This document describes RDF::Trine::Store::Hexastore version 0.135
+This document describes RDF::Trine::Store::Hexastore version 0.138
 
 =head1 SYNOPSIS
 
@@ -28,9 +28,10 @@ use Data::Dumper;
 use RDF::Trine qw(iri);
 use RDF::Trine::Error;
 use List::Util qw(first);
-use List::MoreUtils qw(any mesh);
 use Scalar::Util qw(refaddr reftype blessed);
 use Storable qw(nstore retrieve);
+use Carp qw(croak);
+use Time::HiRes qw ( time );
 
 use constant NODES		=> qw(subject predicate object);
 use constant NODEMAP	=> { subject => 0, predicate => 1, object => 2, context => 3 };
@@ -44,7 +45,7 @@ use constant OTHERNODES	=> {
 
 our $VERSION;
 BEGIN {
-	$VERSION	= "0.135";
+	$VERSION	= "0.138";
 	my $class	= __PACKAGE__;
 	$RDF::Trine::Store::STORE_CLASSES{ $class }	= $VERSION;
 }
@@ -60,8 +61,7 @@ L<RDF::Trine::Store> class.
 
 =item C<< new () >>
 
-Returns a new storage object using the supplied arguments to construct a DBI
-object for the underlying database.
+Returns a new storage object.
 
 =item C<new_with_config ( $hashref )>
 
@@ -108,20 +108,15 @@ The following example initializes a Hexastore store based on a local file and a
 
 sub new {
 	my $class	= shift;
-	my $self	= bless({
-		data		=> $class->_new_index_page,
-		node2id		=> {},
-		id2node		=> {},
-		next_id		=> 1,
-		size		=> 0,
-	}, $class);
+	my $self        = bless({}, $class);
+	$self->nuke; # nuke resets the store, thus doing the same thing as init should do
 	return $self;
 }
 
 sub _new_with_string {
-    my ($self, $config) = @_;
-    my ($filename) = $config =~ m/file=(.+)$/; # TODO: It has a Storable part too, for later use.
-    return $self->load($filename);
+	my ($self, $config) = @_;
+	my ($filename) = $config =~ m/file=(.+)$/; # TODO: It has a Storable part too, for later use.
+	return $self->load($filename);
 }
 
 # TODO: Refactor, almost identical to Memory
@@ -218,11 +213,10 @@ sub get_statements {
 	
 	my @ids		= map { $self->_node2id( $_ ) } @nodes;
 	my @names	= NODES;
-	my @keys	= mesh @names, @ids;
+	my @keys	= map { $names[$_], $ids[$_] } (0 .. $#names);
 	if ($defined == 3) {
 		my $index	= $self->_index_from_pair( $self->_index_root, @keys[ 0,1 ] );
 		my $list	= $self->_index_from_pair( $index, @keys[ 2,3 ] );
-# 		if (any { $_ == $ids[2] } @$list) {
 		if ($self->_page_contains_node( $list, $ids[2] )) {
 			return RDF::Trine::Iterator::Graph->new( [ RDF::Trine::Statement->new( @nodes ) ] );
 		} else {
@@ -279,10 +273,10 @@ sub get_statements {
 		my $rev	= 0;
 		if (@orderby) {
 			$rev	= 1 if ($orderby[1] eq 'DESC');
- 			my $sortkey	= $variable_map{ $orderby[0] };
- 			if ($sortkey ne $ukeys[0]) {
- 				@ukeys	= reverse(@ukeys);
- 			}
+			my $sortkey	= $variable_map{ $orderby[0] };
+			if ($sortkey ne $ukeys[0]) {
+				@ukeys	= reverse(@ukeys);
+			}
 		}
 		
 		my $index	= $self->_index_from_pair( $self->_index_root, @keys );
@@ -293,7 +287,7 @@ sub get_statements {
 		my $ukey1;
 		my $sub		= sub {
 			while (0 == scalar(@local_list)) {
- 				return undef unless (scalar(@ukeys1));
+				return undef unless (scalar(@ukeys1));
 				$ukey1		= shift(@ukeys1);
 #				warn '>>>>>>>>> ' . Dumper( $ukeys[0], $ukey1, $data );
 				my $list	= $self->_index_from_pair( $index, $ukeys[0], $ukey1 );
@@ -330,12 +324,12 @@ sub get_statements {
 		
 		my $rev	= 0;
 		my (@order_keys, $final_key);
- 		if (@orderby) {
+		if (@orderby) {
 			$rev	= 1 if ($orderby[1] eq 'DESC');
- 			my $sortkey	= $variable_map{ $orderby[0] };
- 			my @nodes	= ($sortkey, grep { $_ ne $sortkey } NODES);
- 			@order_keys	= @nodes[0,1];
- 			$final_key	= $nodes[2];
+			my $sortkey	= $variable_map{ $orderby[0] };
+			my @nodes	= ($sortkey, grep { $_ ne $sortkey } NODES);
+			@order_keys	= @nodes[0,1];
+			$final_key	= $nodes[2];
 		} else {
 			$final_key	= 'object';
 			@order_keys	= qw(subject predicate);
@@ -522,7 +516,7 @@ sub _join {
 =cut
 
 sub get_contexts {
-	die;
+	croak "Contexts not supported for the Hexastore store";
 }
 
 =item C<< add_statement ( $statement [, $context] ) >>
@@ -551,6 +545,7 @@ sub add_statement {
 	}
 	if ($added) {
 		$self->{ size }++;
+		$self->{etag} = time;
 	}
 }
 
@@ -586,6 +581,7 @@ sub remove_statement {
 	
 	if ($removed) {
 		$self->{ size }--;
+		$self->{etag} = time;
 	}
 }
 
@@ -593,12 +589,36 @@ sub remove_statement {
 
 Removes the specified C<$statement> from the underlying model.
 
+=item C<< etag >>
+
+Returns an Etag suitable for use in an HTTP Header.
+
 =cut
 
-sub remove_statements {
-	die;
+sub etag {
+	return $_[0]->{etag};
 }
 
+
+=item C<< nuke >>
+
+Permanently removes all the data in the store.
+
+=cut
+
+sub nuke {
+	my $self = shift;
+	$self->{data} = $self->_new_index_page;
+	$self->{node2id} = {};
+	$self->{id2node} = {};
+	$self->{next_id} = 1;
+	$self->{size} = 0;
+	$self->{etag} = time;
+	return $self;
+}
+
+
+
 =item C<< count_statements ($subject, $predicate, $object) >>
 
 Returns a count of all the statements matching the specified subject,
@@ -611,7 +631,7 @@ sub count_statements {
 	my @nodes	= @_;
 	my @ids		= map { $self->_node2id( $_ ) } @nodes;
 	my @names	= NODES;
-	my @keys	= mesh @names, @ids;
+	my @keys	= map { $names[$_], $ids[$_] } (0 .. $#names);
 	my @dkeys;
 	my @ukeys;
 	foreach my $i (0 .. 2) {
@@ -634,7 +654,7 @@ sub count_statements {
 	} else {
 		my $index	= $self->_index_from_pair( $self->_index_root, @keys[ 0,1 ] );
 		my $list	= $self->_index_from_pair( $index, @keys[ 2,3 ] );
-		return ($self->_page_contains_node( $list, $keys[5] ))	# any { $_ == $keys[5] } @$list)
+		return ($self->_page_contains_node( $list, $keys[5] ))
 			? 1
 			: 0;
 	}
@@ -805,7 +825,10 @@ sub _page_contains_node {
 	my $self	= shift;
 	my $list	= shift;
 	my $id		= shift;
-	return (any { $_ == $id } @$list) ? 1 : 0;
+	foreach (@$list) {
+		return 1 if ($_ == $id);
+	}
+	return 0;
 }
 
 sub _add_node_to_page {
